@@ -3,7 +3,6 @@ from __future__ import absolute_import
 import errno
 import functools
 import time
-import weakref
 
 from .. import compat, scheduler
 
@@ -14,7 +13,7 @@ __all__ = ["wait_fds"]
 def wait_fds(fd_events, inmask=1, outmask=2, timeout=None):
     """wait for the first of a number of file descriptors to have activity
 
-    .. note:: this method will block
+    .. note:: this method can block
 
         it will return once there is relevant activity on the file descriptors,
         or the timeout expires
@@ -51,7 +50,7 @@ def wait_fds(fd_events, inmask=1, outmask=2, timeout=None):
             # if there was a timeout then also have to pull
             # the coroutine from the timed_paused structure
             if timeout:
-                scheduler._remove_from_timedout(waketime, current)
+                scheduler._remove_timer(waketime, current)
 
         # in any case, set the event information
         activated.setdefault(fd, 0)
