@@ -2,6 +2,8 @@ from __future__ import absolute_import, with_statement
 
 import contextlib
 import errno
+import fcntl
+import os
 import socket
 import sys
 
@@ -11,11 +13,15 @@ from . import files
 
 __all__ = ["Socket"]
 
+_fcntl = fcntl.fcntl
 _socket = socket.socket
 _socketpair = socket.socketpair
 _fromfd = socket.fromfd
 
-SOCKET_CLOSED = set((errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN))
+_SOCKET_CLOSED = frozenset((errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN))
+_BLOCKING_OP = frozenset((
+        errno.EINPROGRESS, errno.EAGAIN, errno.EWOULDBLOCK, errno.EALREADY))
+_CANT_SEND = frozenset((errno.EWOULDBLOCK, errno.ENOTCONN))
 
 
 class Socket(object):
@@ -27,10 +33,74 @@ class Socket(object):
 
     They provide a totally matching API, however
     """
-    __slots__ = ["_sock", "__weakref__"]
-
     def __init__(self, *args, **kwargs):
-        self._sock = _InnerSocket(*args, **kwargs)
+        sock = kwargs.pop('fromsock', None)
+        if sock is None:
+            sock = socket._realsocket(*args, **kwargs)
+        while hasattr(sock, "_sock"):
+            sock = sock._sock
+        self._sock = sock
+
+        # copy over attributes
+        self._fileno = sock.fileno()
+        self._timeout = sock.gettimeout()
+        self._closed = False
+
+        # make the underlying socket non-blocking
+        fl = _fcntl(self._fileno, fcntl.F_GETFL)
+        if 0 == fl & os.O_NONBLOCK:
+            _fcntl(self._fileno, fcntl.F_SETFL, fl | os.O_NONBLOCK)
+
+        # but by default, it blocks greenlets
+        self._blocking = True
+
+        # create events
+        self._readable = util.Event()
+        self._writable = util.Event()
+
+        # register the socket for this fd in the scheduler
+        scheduler._register_fd(
+                self._fileno, self._on_readable, self._on_writable)
+
+    def _on_readable(self):
+        self._readable.set()
+        self._readable.clear()
+
+    def _on_writable(self):
+        self._writable.set()
+        self._writable.clear()
+
+    def _poller_evmask(self, poller, events):
+        mask = 0
+        if 'r' in events:
+            mask |= poller.INMASK
+        if 'w' in events:
+            mask |= poller.OUTMASK
+        if 'e' in events:
+            mask |= poller.ERRMASK
+        return mask
+
+    @contextlib.contextmanager
+    def _registered(self, events=None):
+        poller = scheduler.state.poller
+        if events:
+            events = self._poller_evmask(poller, events)
+        try:
+            counter = poller.register(self, events)
+        except EnvironmentError, exc:
+            if exc.args and exc.args[0] in errno.errorcode:
+                raise socket.error(*exc.args)
+            raise
+
+        try:
+            yield
+        finally:
+            try:
+                poller.unregister(self, counter)
+            except EnvironmentError, exc:
+                if exc.args and exc.args[0] in errno.errorcode:
+                    raise socket.error(*exc.args)
+                raise
 
     @property
     def family(self):
@@ -44,24 +114,6 @@ class Socket(object):
     def type(self):
         return self._sock.type
 
-    def close(self):
-        """close the current connection on the socket
-
-        After this point all operations attempted on this socket will fail, and
-        once any queued data is flushed, the remote end will not receive any
-        more data
-        """
-        self._sock = socket._closedsocket()
-
-    def dup(self):
-        """create a new copy of the current socket on the same file descriptor
-
-        :returns: a new :class:`Socket`
-        """
-        copy = object.__new__(type(self))
-        copy._sock = self._sock.dup()
-        return copy
-
     def accept(self):
         """accept a connection on the host/port to which the socket is bound
 
@@ -74,10 +126,21 @@ class Socket(object):
             a two-tuple of ``(socket, address)`` where the socket is connected,
             and the address is the ``(ip_address, port)`` of the remote end
         """
-        innersock, address = self._sock.accept()
-        client = object.__new__(type(self))
-        client._sock = innersock
-        return client, address
+        with self._registered('re'):
+            while 1:
+                try:
+                    client, addr = self._sock.accept()
+                except socket.error, exc:
+                    if not self._blocking or exc[0] not in _BLOCKING_OP:
+                        raise
+                    sys.exc_clear()
+                    if self._readable.wait(self.gettimeout()):
+                        raise socket.timeout("timed out")
+                    if scheduler.state.interrupted:
+                        raise IOError(errno.EINTR,
+                                "interrupted system call")
+                    continue
+                return type(self)(fromsock=client), addr
 
     def bind(self, address):
         """set the socket to operate on an address
@@ -89,6 +152,16 @@ class Socket(object):
         """
         return self._sock.bind(address)
 
+    def close(self):
+        """close the current connection on the socket
+
+        After this point all operations attempted on this socket will fail, and
+        once any queued data is flushed, the remote end will not receive any
+        more data
+        """
+        self._closed = True
+        self._sock = socket._closedsocket()
+
     def connect(self, address):
         """initiate a new connection to a remote socket bound to an address
 
@@ -99,7 +172,19 @@ class Socket(object):
             depends on the socket's type; for TCP sockets, this is a
             ``(host, port``) two-tuple
         """
-        return self._sock.connect(address)
+        address = _dns_resolve(self, address)
+        with self._registered('we'):
+            while 1:
+                err = self._sock.connect_ex(address)
+                if not self._blocking or err not in _BLOCKING_OP:
+                    if err not in (0, errno.EISCONN):
+                        raise socket.error(err, errno.errorcode[err])
+                    return
+                if self._writable.wait(self.gettimeout()):
+                    raise socket.timeout("timed out")
+                if scheduler.state.interrupted:
+                    raise IOError(errno.EINTR,
+                            "interrupted system call")
 
     def connect_ex(self, address):
         """initiate a connection without blocking
@@ -112,14 +197,21 @@ class Socket(object):
         :returns:
             the error code for the connection attempt -- 0 indicates success
         """
-        return self._sock.connect_ex(address)
+        return self._sock.connect_ex(_dns_resolve(self, address))
+
+    def dup(self):
+        """create a new copy of the current socket on the same file descriptor
+
+        :returns: a new :class:`Socket`
+        """
+        return type(self)(fromsock=self._sock.dup())
 
     def fileno(self):
         """get the file descriptor
 
         :returns: the integer file descriptor of the socket
         """
-        return self._sock.fileno()
+        return self._fileno
 
     def getpeername(self):
         """address information for the remote end of a connection
@@ -167,7 +259,7 @@ class Socket(object):
             the number of seconds the socket's blocking operations should block
             before raising a ``socket.timeout`` in a float value
         """
-        return self._sock.gettimeout()
+        return self._timeout
 
     def listen(self, backlog):
         """listen for connections made to the socket
@@ -195,7 +287,7 @@ class Socket(object):
             a file-like object for which reading and writing sends and receives
             data over the socket connection
         """
-        return self._sock.makefile(mode, bufsize)
+        return SocketFile(self._sock, mode)
 
     def recv(self, bufsize, flags=0):
         """receive data from the connection
@@ -215,7 +307,23 @@ class Socket(object):
 
         :returns: the data it read from the socket connection
         """
-        return self._sock.recv(bufsize, flags)
+        with self._registered('re'):
+            while 1:
+                if self._closed:
+                    raise socket.error(errno.EBADF, "Bad file descriptor")
+                try:
+                    return self._sock.recv(bufsize, flags)
+                except socket.error, exc:
+                    if not self._blocking or exc[0] not in _BLOCKING_OP:
+                        if exc[0] in _SOCKET_CLOSED:
+                            self._closed = True
+                            return ''
+                        raise
+                    sys.exc_clear()
+                    if self._readable.wait(self.gettimeout()):
+                        raise socket.timeout("timed out")
+                    if scheduler.state.interrupted:
+                        raise IOError(errno.EINTR, "interrupted system call")
 
     def recv_into(self, buffer, bufsize=-1, flags=0):
         """receive data from the connection and place it into a buffer
@@ -236,7 +344,23 @@ class Socket(object):
 
         :returns: the number of bytes received and placed in the buffer
         """
-        return self._sock.recv_into(buffer, bufsize, flags)
+        with self._registered('re'):
+            while 1:
+                if self._closed:
+                    raise socket.error(errno.EBADF, "Bad file descriptor")
+                try:
+                    return self._sock.recv_into(buffer, bufsize, flags)
+                except socket.error, exc:
+                    if not self._blocking or exc[0] not in _BLOCKING_OP:
+                        if exc[0] in _SOCKET_CLOSED:
+                            self._closed = True
+                            return
+                        raise
+                    sys.exc_clear()
+                    if self._readable.wait(self.gettimeout()):
+                        raise socket.timeout("timed out")
+                    if scheduler.state.interrupted:
+                        raise IOError(errno.EINTR, "interrupted system call")
 
     def recvfrom(self, bufsize, flags=0):
         """receive data on a socket that isn't necessarily a 1-1 connection
@@ -256,7 +380,23 @@ class Socket(object):
             a two-tuple of ``(data, address)`` -- the string data received and
             the address from which it was received
         """
-        return self._sock.recvfrom(bufsize, flags)
+        with self._registered('re'):
+            while 1:
+                if self._closed:
+                    raise socket.error(errno.EBADF, "Bad file descriptor")
+                try:
+                    return self._sock.recvfrom(bufsize, flags)
+                except socket.error, exc:
+                    if not self._blocking or exc[0] not in _BLOCKING_OP:
+                        if exc[0] in _SOCKET_CLOSED:
+                            self._closed = True
+                            return '', (None, 0)
+                        raise
+                    sys.exc_clear()
+                    if self._readable.wait(self.gettimeout()):
+                        raise socket.timeout("timed out")
+                    if scheduler.state.interrupted:
+                        raise IOError(errno.EINTR, "interrupted system call")
 
     def recvfrom_into(self, buffer, bufsize=-1, flags=0):
         """receive data on a non-TCP socket and place it in a buffer
@@ -279,7 +419,23 @@ class Socket(object):
             a two-tuple of ``(bytes, address)`` -- the number of bytes received
             and placed in the buffer, and the address it was received from
         """
-        return self._sock.recvfrom_into(buffer, bufsize, flags)
+        with self._registered('re'):
+            while 1:
+                if self._closed:
+                    raise socket.error(errno.EBADF, "Bad file descriptor")
+                try:
+                    return self._sock.recvfrom_into(buffer, bufsize, flags=0)
+                except socket.error, exc:
+                    if not self._blocking or exc[0] not in _BLOCKING_OP:
+                        if exc[0] in _SOCKET_CLOSED:
+                            self._closed = True
+                            return '', (None, 0)
+                        raise
+                    sys.exc_clear()
+                    if self._readable.wait(self.gettimeout()):
+                        raise socket.timeout("timed out")
+                    if scheduler.state.interrupted:
+                        raise IOError(errno.EINTR, "interrupted system call")
 
     def send(self, data, flags=0):
         """send data over the socket connection
@@ -297,7 +453,12 @@ class Socket(object):
             the number of bytes successfully sent, which may not necessarily be
             all the provided data
         """
-        return self._sock.send(data, flags)
+        try:
+            return self._sock.send(data)
+        except socket.error, exc:
+            if exc[0] not in _CANT_SEND:
+                raise
+            return 0
 
     def sendall(self, data, flags=0):
         """send data over the connection, and keep sending until it all goes
@@ -311,7 +472,15 @@ class Socket(object):
             :meth:`recv`
         :type flags: int
         """
-        return self._sock.sendall(data, flags)
+        with self._registered('we'):
+            sent = self.send(data, flags)
+            while sent < len(data):
+                if self._blocking:
+                    if self._writable.wait(self.gettimeout()):
+                        raise socket.timeout("timed out")
+                    if scheduler.state.interrupted:
+                        raise IOError(errno.EINTR, "interrupted system call")
+                sent += self.send(data[sent:], flags)
 
     def sendto(self, data, *args):
         """send data to a particular address
@@ -328,7 +497,12 @@ class Socket(object):
             a representation of the address to which to send the data, the
             format depends on the socket's type
         """
-        return self._sock.sendto(data, *args)
+        try:
+            return self._sock.sendto(data, *args)
+        except socket.error, exc:
+            if exc[0] not in _CANT_SEND:
+                raise
+            return 0
 
     def setblocking(self, flag):
         """modify the behavior of blocking methods on the socket
@@ -344,7 +518,7 @@ class Socket(object):
         :param flag: whether to enable or disable blocking
         :type flag: bool
         """
-        return self._sock.setblocking(flag)
+        self._blocking = bool(flag)
 
     def setsockopt(self, level, optname, value):
         """set the value of a given socket option
@@ -380,287 +554,20 @@ class Socket(object):
             before raising a ``socket.timeout``
         :type timeout: float or None
         """
-        return self._sock.settimeout(timeout)
-
-
-def socket_fromfd(fd, family, type_, *args):
-    raw_sock = socket.fromfd(fd, family, type_, *args)
-    return Socket(fromsock=raw_sock)
-
-
-class _InnerSocket(object):
-    def __init__(self, *args, **kwargs):
-        # wrap a basic socket or build our own
-        sock = kwargs.pop('fromsock', None) or _socket(*args, **kwargs)
-        while hasattr(sock, "_sock"):
-            sock = sock._sock
-        self._sock = sock
-
-        # copy over attributes
-        self.family = self._sock.family
-        self.type = self._sock.type
-        self.proto = self._sock.proto
-        self._fileno = self._sock.fileno()
-
-        # some more housekeeping
-        tout = sock.gettimeout()
-        self._timeout = tout
-        self._closed = False
-
-        # make the underlying socket non-blocking
-        self._sock.setblocking(False)
-        self._blocking = True
-
-        # create events
-        self._readable = util.Event()
-        self._writable = util.Event()
-
-        # allow for lookup by fileno
-        scheduler._register_fd(
-                self._fileno, self._on_readable, self._on_writable)
-
-    def _on_readable(self):
-        self._readable.set()
-        self._readable.clear()
-
-    def _on_writable(self):
-        self._writable.set()
-        self._writable.clear()
-
-    @contextlib.contextmanager
-    def _registered(self, events=None):
-        poller = scheduler.state.poller
-        mask = None
-        if events:
-            mask = 0
-            if 'r' in events:
-                mask |= poller.INMASK
-            if 'w' in events:
-                mask |= poller.OUTMASK
-            if 'e' in events:
-                mask |= poller.ERRMASK
-        try:
-            counter = poller.register(self, mask)
-        except EnvironmentError, error:
-            if error.args and error.args[0] in errno.errorcode:
-                raise socket.error(*error.args)
-            raise
-
-        try:
-            yield
-        finally:
-            try:
-                poller.unregister(self, counter)
-            except EnvironmentError, error:
-                if error.args and error.args[0] in errno.errorcode:
-                    raise socket.error(*error.args)
-                raise
-
-    def accept(self):
-        with self._registered('re'):
-            while 1:
-                try:
-                    client, addr = self._sock.accept()
-                except socket.error, err:
-                    if self._blocking and err[0] in (
-                            errno.EAGAIN, errno.EWOULDBLOCK):
-                        sys.exc_clear()
-                        if self._readable.wait(self.gettimeout()):
-                            raise socket.timeout("timed out")
-                        if scheduler.state.interrupted:
-                            raise IOError(errno.EINTR,
-                                    "interrupted system call")
-                        continue
-                    else:
-                        raise
-                return type(self)(fromsock=client), addr
-
-    def bind(self, address):
-        return self._sock.bind(address)
-
-    def close(self):
-        self._closed = True
-        self._sock.close()
-
-    def connect(self, address):
-        address = _dns_resolve(self, address)
-        with self._registered('we'):
-            while True:
-                err = self.connect_ex(address)
-                if self._blocking and err in (
-                        errno.EINPROGRESS, errno.EALREADY, errno.EWOULDBLOCK):
-                    if self._writable.wait(self.gettimeout()):
-                        raise socket.timeout("timed out")
-                    if scheduler.state.interrupted:
-                        raise IOError(errno.EINTR,
-                                "interrupted system call")
-                    continue
-                if err not in (0, errno.EISCONN):
-                    raise socket.error(err, errno.errorcode[err])
-                return
-
-    def connect_ex(self, address):
-        return self._sock.connect_ex(_dns_resolve(self, address))
-
-    def dup(self):
-        return type(self)(fromsock=self._sock.dup())
-
-    def fileno(self):
-        return self._fileno
-
-    def getpeername(self):
-        return self._sock.getpeername()
-
-    def getsockname(self):
-        return self._sock.getsockname()
-
-    def getsockopt(self, *args):
-        return self._sock.getsockopt(*args)
-
-    def gettimeout(self):
-        return self._timeout
-
-    def listen(self, backlog):
-        return self._sock.listen(backlog)
-
-    def makefile(self, mode='r', bufsize=-1):
-        return SocketFile(self, mode)
-
-    def recv(self, nbytes, flags=0):
-        with self._registered('re'):
-            while 1:
-                if self._closed:
-                    raise socket.error(errno.EBADF, "Bad file descriptor")
-                try:
-                    return self._sock.recv(nbytes, flags)
-                except socket.error, e:
-                    if self._blocking and e[0] in (
-                            errno.EWOULDBLOCK, errno.EAGAIN):
-                        sys.exc_clear()
-                        if self._readable.wait(self.gettimeout()):
-                            raise socket.timeout("timed out")
-                        if scheduler.state.interrupted:
-                            raise IOError(errno.EINTR,
-                                    "interrupted system call")
-                        continue
-                    if e[0] in SOCKET_CLOSED:
-                        self._closed = True
-                        return ''
-                    raise
-
-    def recv_into(self, buffer, nbytes=0, flags=0):
-        with self._registered('re'):
-            while 1:
-                if self._closed:
-                    raise socket.error(errno.EBADF, "Bad file descriptor")
-                try:
-                    return self._sock.recv_into(buffer, nbytes, flags)
-                except socket.error, e:
-                    if self._blocking and e[0] in (
-                            errno.EWOULDBLOCK, errno.EAGAIN):
-                        sys.exc_clear()
-                        if self._readable.wait(self.gettimeout()):
-                            raise socket.timeout("timed out")
-                        if scheduler.state.interrupted:
-                            raise IOError(errno.EINTR,
-                                    "interrupted system call")
-                        continue
-                    if e[0] in SOCKET_CLOSED:
-                        self._closed = True
-                        return
-                    raise
-
-    def recvfrom(self, nbytes, flags=0):
-        with self._registered('re'):
-            while 1:
-                if self._closed:
-                    raise socket.error(errno.EBADF, "Bad file descriptor")
-                try:
-                    return self._sock.recvfrom(nbytes, flags)
-                except socket.error, e:
-                    if self._blocking and e[0] in (
-                            errno.EWOULDBLOCK, errno.EAGAIN):
-                        sys.exc_clear()
-                        if self._readable.wait(self.gettimeout()):
-                            raise socket.timeout("timed out")
-                        if scheduler.state.interrupted:
-                            raise IOError(errno.EINTR,
-                                    "interrupted system call")
-                        continue
-                    if e[0] in SOCKET_CLOSED:
-                        self._closed = True
-                        return '', (None, 0)
-                    raise
-
-    def recvfrom_into(self, buffer, nbytes=0, flags=0):
-        with self._registered('re'):
-            while 1:
-                if self._closed:
-                    raise socket.error(errno.EBADF, "Bad file descriptor")
-                try:
-                    return self._sock.recvfrom_into(buffer, nbytes, flags=0)
-                except socket.error, e:
-                    if self._blocking and e[0] in (
-                            errno.EWOULDBLOCK, errno.EAGAIN):
-                        sys.exc_clear()
-                        if self._readable.wait(self.gettimeout()):
-                            raise socket.timeout("timed out")
-                        if scheduler.state.interrupted:
-                            raise IOError(errno.EINTR,
-                                    "interrupted system call")
-                        continue
-                    if e[0] in SOCKET_CLOSED:
-                        self._closed = True
-                        return '', (None, 0)
-                    raise
-
-    def send(self, data, flags=0):
-        try:
-            return self._sock.send(data)
-        except socket.error, err:
-            if err[0] in (errno.EWOULDBLOCK, errno.ENOTCONN):
-                return 0
-            raise
-
-    def sendall(self, data, flags=0):
-        with self._registered('we'):
-            sent = self.send(data, flags)
-            while sent < len(data):
-                if self._blocking:
-                    if self._writable.wait(self.gettimeout()):
-                        raise socket.timeout("timed out")
-                    if scheduler.state.interrupted:
-                        raise IOError(errno.EINTR,
-                                "interrupted system call")
-                sent += self.send(data[sent:], flags)
-
-    def sendto(self, *args):
-        try:
-            return self._sock.sendto(*args)
-        except socket.error, err:
-            if err[0] in (errno.EWOULDBLOCK, errno.ENOTCONN):
-                return 0
-            raise
-
-    def setblocking(self, flag):
-        self._blocking = bool(flag)
-
-    def setsockopt(self, level, option, value):
-        return self._sock.setsockopt(level, option, value)
-
-    def shutdown(self, flag):
-        return self._sock.shutdown(flag)
-
-    def settimeout(self, timeout):
         if timeout is not None:
             timeout = float(timeout)
         self._timeout = timeout
 
 
+def socket_fromfd(fd, family, type_, *args):
+    raw_sock = _fromfd(fd, family, type_, *args)
+    return Socket(fromsock=raw_sock)
+
+
 class SocketFile(files.FileBase):
     def __init__(self, sock, mode='b', bufsize=-1):
         super(SocketFile, self).__init__()
-        self._sock = sock
+        self._sock = Socket(fromsock=sock)
         self.mode = mode
         if bufsize > 0:
             self.CHUNKSIZE = bufsize
